from pathlib import Path
from typing import Protocol

from pydantic import BaseModel


class ExtractTarget(BaseModel):
    key: str
    description: str = ""
    constraint: str = ""


class Item(BaseModel):
    key: str
    values: list[str]


class ITextExtractor(Protocol):
    def extract(self, file_path: Path, lang: str = "") -> list[str]:
        ...


class IItemExtractor(Protocol):
    def extract_from_text(self, texts: list[str], targets: list[ExtractTarget]) -> list[Item]:
        ...



import logging


def setup_logger(name: str, level: str = "DEBUG", logfile_path: str | None = None) -> logging.Logger:
    logger = logging.getLogger(name)
    logger.setLevel(logging.DEBUG)

    handlers: list[logging.Handler] = [logging.StreamHandler()]

    if logfile_path is not None:
        handlers.append(logging.FileHandler(logfile_path))

    formatter = logging.Formatter("%(asctime)s - %(levelname)s - %(filename)s:%(funcName)s:%(lineno)s - %(message)s")

    for handler in handlers:
        handler.setLevel(getattr(logging, level, logging.NOTSET))
        handler.setFormatter(formatter)
        logger.addHandler(handler)

    return logger



from pathlib import Path
from typing import Any

import pyocr
from google.cloud import vision
from PIL import Image
from pyocr.builders import TextBuilder


class LambdaTextExtractor:
    @staticmethod
    def extract(file_path: Path, lang: str = "") -> list[str]:
        return file_path.read_text().split("\n")


class GoogleVisionTextExtractor:
    def __init__(self):
        self.client = vision.ImageAnnotatorClient()

    def extract(self, file_path: Path, lang: str = "") -> list[str]:
        image = vision.Image(content=file_path.read_bytes())
        response = self.client.document_text_detection(image=image)
        return response.full_text_annotation.text.split("\n")


class TesseractTextExtractor:
    def __init__(self) -> None:
        tools: list[Any] = pyocr.get_available_tools()
        if len(tools) == 0:
            raise RuntimeError("Tesseract does not configured correctly.")

        self.tool = tools[0]
        assert hasattr(self.tool, "image_to_string")

        self.builder = TextBuilder(tesseract_layout=6)

    def extract(self, file_path: Path, lang: str = "Japanese") -> list[str]:
        image = Image.open(file_path)
        text: str = self.tool.image_to_string(image, lang=lang, builder=self.builder)
        return text.split("\n")



import json
from typing import Any

from langchain import PromptTemplate
from langchain.llms import OpenAIChat

from .logger import setup_logger
from .schema import ExtractTarget, Item

logger = setup_logger(__name__)


class GPTItemExtractor:
    TEMPLATE = """Carry out the following instructions.
* Correct any OCR errors in the input text
* Extract the following items according to the given constraints
|Item|Description|Constraint|
{extract_targets}
* Review the extracted results and point out any errors
* If there are any errors, correct them and output the extraction results again
* OUTPUT the extraction results in JSON format, enclosed in triple backticks
  (set the key as string type and the value as array type)

The text is as follows.
{text}
"""

    def __init__(self, model_name: str = "gpt-3.5-turbo") -> None:
        self.gpt = OpenAIChat(model_name=model_name, temperature=0)  # type: ignore
        self.prompt = PromptTemplate(input_variables=["extract_targets", "text"], template=self.TEMPLATE)

    def extract_from_text(self, texts: list[str], targets: list[ExtractTarget]) -> list[Item]:
        extract_targets = "\n".join([f"|{t.key}|{t.description}|{t.constraint}|" for t in targets])

        inputs = self.prompt.format(extract_targets=extract_targets, text="\n".join(texts))
        logger.info(inputs)
        outputs = self.gpt(inputs)
        logger.info(outputs)

        results: dict[str, Any]
        try:
            *_, json_output, _ = outputs.split("```")
            results = json.loads(json_output.replace("json", ""))
        except Exception:
            logger.exception(self.__class__.__name__)
            results = {}
        return [Item(key=target.key, values=results.get(target.key, [])) for target in targets]



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['xtracture']

package_data = \
{'': ['*']}

install_requires = \
['google-cloud-vision>=3.4.0,<4.0.0',
 'langchain>=0.0.117,<0.0.118',
 'openai>=0.27.2,<0.28.0',
 'pyocr>=0.8.3,<0.9.0']

setup_kwargs = {
    'name': 'xtracture',
    'version': '0.4.0',
    'description': 'Xtracture is an open source library designed to efficiently extract arbitrary elements from documents.',
    'long_description': '# Xtracture: Open Source Document Content Extractuion Library\n\nXtracture is an open source library designed to efficiently extract arbitrary elements from documents.\n\n## Features\n\n- Natural language rule creation using LLMs\n- Switchable OCR engines for optimized perfomance and accuracy\n\n## prerequirements\n\n- OpenAI API Key (for LLM rule creation)\n\n## Installation\n\n```\npip install -U xtracture\n```\n\n## Usage\n\n### Use Google Cloud Vision API\n\nGoogle CLoud Vision Credentials must be correctly configured.\n\nsee `examples/google_cloud_vision_example.py`.\n\n### Use Tesseract\n\nTesseract must be installed beforehand.\n\nsee `examples/tesseract_example.py`.\n\n### Use only GPT Extractor\n\nYou can input OCR-processed text file.\nsee `examples/lambda_example.py`.\n\n## License\n\nXtracture is released under the MIT License.\n',
    'author': 'ryo.ishii',
    'author_email': 'ryoishii1101@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



from pathlib import Path

from .schema import ExtractTarget, IItemExtractor, Item, ITextExtractor


class Extractor:
    def __init__(self, text_extractor: ITextExtractor, item_extractor: IItemExtractor) -> None:
        self.text_extractor = text_extractor
        self.item_extractor = item_extractor

    def extract(self, file_path: Path, targets: list[ExtractTarget]) -> list[Item]:
        texts = self.text_extractor.extract(file_path)
        return self.item_extractor.extract_from_text(texts, targets)
